from typing import List, TypedDict

from identify_browser import BrowserIdentityDict, identify_browser


class BrowserTestDict(TypedDict):
    actual_name: str
    actual_version: str
    actual_platform: str
    user_agent: str
    sec_ch_ua: str
    sec_ch_ua_platform: str
    expected_identity: BrowserIdentityDict


def test_identify_browser() -> None:
    browsers: List[BrowserTestDict] = [
        # Can be correctly identified
        {
            "actual_name": "Aloha",
            "actual_version": "2",
            "actual_platform": "Windows",
            "user_agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/123.0.0.0 AlohaBrowser/2.0.0.0 Safari/537.36",
            "sec_ch_ua": '"Chromium";v="123", "Not:A-Brand";v="8"',
            "sec_ch_ua_platform": '"Windows"',
            "expected_identity": {
                "name": "Aloha",
                "version": "2",
                "platform": "Windows",
            },
        },
        {
            "actual_name": "Aloha",
            "actual_version": "6",
            "actual_platform": "Android",
            "user_agent": "Mozilla/5.0 (Linux; Android 10; K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/123.0.0.0 Mobile Safari/537.36 AlohaBrowser/6.0.2",
            "sec_ch_ua": '"Not_A Brand";v="8", "Chromium";v="123", "Android WebView";v="123"',
            "sec_ch_ua_platform": '"Android"',
            "expected_identity": {
                "name": "Aloha",
                "version": "6",
                "platform": "Android",
            },
        },
        {
            "actual_name": "Aloha",
            "actual_version": "5",
            "actual_platform": "iOS",
            "user_agent": "Mozilla/5.0 (iPhone; CPU iPhone OS 17_5_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Version/17.5.1 Safari/605.1.15 AlohaBrowser/5.11.0",
            "sec_ch_ua": "",
            "sec_ch_ua_platform": "",
            "expected_identity": {
                "name": "Aloha",
                "version": "5",
                "platform": "iOS",
            },
        },
        {
            "actual_name": "Avast Secure Browser",
            "actual_version": "8.0.0",
            "actual_platform": "Android",
            "user_agent": "Mozilla/5.0 (Linux; Android 14; xToyQdFTiQ; U; en) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/124.0.6367.42 Mobile Safari/537.36",
            "sec_ch_ua": '"Avast Secure Browser";v="8.0.0", "Not-A.Brand";v="99", "Chromium";v="124"',
            "sec_ch_ua_platform": '"Android"',
            "expected_identity": {
                "name": "Avast Secure Browser",
                "version": "8.0.0",
                "platform": "Android",
            },
        },
        {
            "actual_name": "Avast Secure Browser",
            "actual_version": "5",
            "actual_platform": "iOS",
            "user_agent": "Mozilla/5.0 (iPhone; CPU iPhone OS 17_5_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) AvastSecureBrowser/5.7.0 Mobile/15E148 Version/17.0 Safari/605.1.15",
            "sec_ch_ua": "",
            "sec_ch_ua_platform": "",
            "expected_identity": {
                "name": "Avast Secure Browser",
                "version": "5",
                "platform": "iOS",
            },
        },
        {
            "actual_name": "AVG Secure Browser",
            "actual_version": "8.0.0",
            "actual_platform": "Android",
            "user_agent": "Mozilla/5.0 (Linux; Android 14; QChq6Q5PJU; U; en) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/124.0.6367.42 Mobile Safari/537.36",
            "sec_ch_ua": '"AVG Secure Browser";v="8.0.0", "Not-A.Brand";v="99", "Chromium";v="124"',
            "sec_ch_ua_platform": '"Android"',
            "expected_identity": {
                "name": "AVG Secure Browser",
                "version": "8.0.0",
                "platform": "Android",
            },
        },
        {
            "actual_name": "Basilisk",
            "actual_version": "102",
            "actual_platform": "Windows",
            "user_agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:102.0) Gecko/20100101 Goanna/6.6 Firefox/102.0 Basilisk/20240511",
            "sec_ch_ua": "",
            "sec_ch_ua_platform": "",
            "expected_identity": {
                "name": "Basilisk",
                "version": "102",
                "platform": "Windows",
            },
        },
        {
            "actual_name": "Basilisk",
            "actual_version": "102",
            "actual_platform": "macOS",
            "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 14.3; rv:102.0) Gecko/20100101 Goanna/6.6 Firefox/102.0 Basilisk/20240511",
            "sec_ch_ua": "",
            "sec_ch_ua_platform": "",
            "expected_identity": {
                "name": "Basilisk",
                "version": "102",
                "platform": "macOS",
            },
        },
        {
            "actual_name": "Basilisk",
            "actual_version": "102",
            "actual_platform": "Linux",
            "user_agent": "Mozilla/5.0 (X11; Linux x86_64; rv:102.0) Gecko/20100101 Goanna/6.6 Firefox/102.0 Basilisk/20240510",
            "sec_ch_ua": "",
            "sec_ch_ua_platform": "",
            "expected_identity": {
                "name": "Basilisk",
                "version": "102",
                "platform": "Linux",
            },
        },
        {
            "actual_name": "Brave",
            "actual_version": "126",
            "actual_platform": "Windows",
            "user_agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/126.0.0.0 Safari/537.36",
            "sec_ch_ua": '"Not/A)Brand";v="8", "Chromium";v="126", "Brave";v="126"',
            "sec_ch_ua_platform": '"Windows"',
            "expected_identity": {
                "name": "Brave",
                "version": "126",
                "platform": "Windows",
            },
        },
        {
            "actual_name": "Brave",
            "actual_version": "121",
            "actual_platform": "macOS",
            "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/121.0.0.0 Safari/537.36",
            "sec_ch_ua": '"Not A(Brand";v="99", "Brave";v="121", "Chromium";v="121"',
            "sec_ch_ua_platform": '"macOS"',
            "expected_identity": {
                "name": "Brave",
                "version": "121",
                "platform": "macOS",
            },
        },
        {
            "actual_name": "Brave",
            "actual_version": "126",
            "actual_platform": "Linux",
            "user_agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/126.0.0.0 Safari/537.36",
            "sec_ch_ua": '"Not/A)Brand";v="8", "Chromium";v="126", "Brave";v="126"',
            "sec_ch_ua_platform": '"Linux"',
            "expected_identity": {
                "name": "Brave",
                "version": "126",
                "platform": "Linux",
            },
        },
        {
            "actual_name": "Brave",
            "actual_version": "125",
            "actual_platform": "Android",
            "user_agent": "Mozilla/5.0 (Linux; Android 10; K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/125.0.0.0 Mobile Safari/537.36",
            "sec_ch_ua": '"Brave";v="125", "Chromium";v="125", "Not.A/Brand";v="24"',
            "sec_ch_ua_platform": '"Android"',
            "expected_identity": {
                "name": "Brave",
                "version": "125",
                "platform": "Android",
            },
        },
        {
            "actual_name": "DuckDuckGo",
            "actual_version": "124",
            "actual_platform": "Windows",
            "user_agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/124.0.0.0 Safari/537.36 Edg/124.0.0.0",
            "sec_ch_ua": '"Chromium";v="124", "DuckDuckGo";v="124", "Not-A.Brand";v="99"',
            "sec_ch_ua_platform": '"Windows"',
            "expected_identity": {
                "name": "DuckDuckGo",
                "version": "124",
                "platform": "Windows",
            },
        },
        {
            "actual_name": "DuckDuckGo",
            "actual_version": "17",
            "actual_platform": "macOS",
            "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/17.3.1 Safari/605.1.15 Ddg/17.3.1",
            "sec_ch_ua": "",
            "sec_ch_ua_platform": "",
            "expected_identity": {
                "name": "DuckDuckGo",
                "version": "17",
                "platform": "macOS",
            },
        },
        {
            "actual_name": "DuckDuckGo",
            "actual_version": "125",
            "actual_platform": "Android",
            "user_agent": "Mozilla/5.0 (Linux; Android 10; K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/125.0.0.0 Mobile Safari/537.36",
            "sec_ch_ua": '"DuckDuckGo";v="125", "Chromium";v="125", "Not.A/Brand";v="24"',
            "sec_ch_ua_platform": '"Android"',
            "expected_identity": {
                "name": "DuckDuckGo",
                "version": "125",
                "platform": "Android",
            },
        },
        {
            "actual_name": "DuckDuckGo",
            "actual_version": "17",
            "actual_platform": "iOS",
            "user_agent": "Mozilla/5.0 (iPhone; CPU iPhone OS 17_5_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/17.5 Mobile/15E148 Safari/604.1 Ddg/17.5",
            "sec_ch_ua": "",
            "sec_ch_ua_platform": "",
            "expected_identity": {
                "name": "DuckDuckGo",
                "version": "17",
                "platform": "iOS",
            },
        },
        {
            "actual_name": "Ecosia",
            "actual_version": "119",
            "actual_platform": "Android",
            "user_agent": "Mozilla/5.0 (Linux; Android 10; K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/119.0.0.0 Mobile Safari/537.36 (Ecosia android@119)",
            "sec_ch_ua": '"Chromium";v="119", "Not?A_Brand";v="24"',
            "sec_ch_ua_platform": '"Android"',
            "expected_identity": {
                "name": "Ecosia",
                "version": "119",
                "platform": "Android",
            },
        },
        {
            "actual_name": "Ecosia",
            "actual_version": "9",
            "actual_platform": "iOS",
            "user_agent": "Mozilla/5.0 (iPhone; CPU iPhone OS 17_5_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/17.5.1 Mobile/15E148 Safari/604.1 (Ecosia ios@9.5.0.1569)",
            "sec_ch_ua": "",
            "sec_ch_ua_platform": "",
            "expected_identity": {
                "name": "Ecosia",
                "version": "9",
                "platform": "iOS",
            },
        },
        {
            "actual_name": "Firefox",
            "actual_version": "127",
            "actual_platform": "Windows",
            "user_agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:127.0) Gecko/20100101 Firefox/127.0",
            "sec_ch_ua": "",
            "sec_ch_ua_platform": "",
            "expected_identity": {
                "name": "Firefox",
                "version": "127",
                "platform": "Windows",
            },
        },
        {
            "actual_name": "Firefox",
            "actual_version": "127",
            "actual_platform": "macOS",
            "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.15; rv:127.0) Gecko/20100101 Firefox/127.0",
            "sec_ch_ua": "",
            "sec_ch_ua_platform": "",
            "expected_identity": {
                "name": "Firefox",
                "version": "127",
                "platform": "macOS",
            },
        },
        {
            "actual_name": "Firefox",
            "actual_version": "118",
            "actual_platform": "Linux",
            "user_agent": "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:109.0) Gecko/20100101 Firefox/118.0",
            "sec_ch_ua": "",
            "sec_ch_ua_platform": "",
            "expected_identity": {
                "name": "Firefox",
                "version": "118",
                "platform": "Ubuntu",
            },
        },
        {
            "actual_name": "Firefox",
            "actual_version": "127",
            "actual_platform": "Android",
            "user_agent": "Mozilla/5.0 (Android 14; Mobile; rv:127.0) Gecko/127.0 Firefox/127.0",
            "sec_ch_ua": "",
            "sec_ch_ua_platform": "",
            "expected_identity": {
                "name": "Firefox",
                "version": "127",
                "platform": "Android",
            },
        },
        {
            "actual_name": "Firefox",
            "actual_version": "127",
            "actual_platform": "iOS",
            "user_agent": "Mozilla/5.0 (iPhone; CPU iPhone OS 17_5_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) FxiOS/127.0  Mobile/15E148 Safari/605.1.15",
            "sec_ch_ua": "",
            "sec_ch_ua_platform": "",
            "expected_identity": {
                "name": "Firefox",
                "version": "127",
                "platform": "iOS",
            },
        },
        {
            "actual_name": "Ghostery",
            "actual_version": "3",
            "actual_platform": "Android",
            "user_agent": "Mozilla/5.0 (Android 14; Mobile; rv:125.0; Ghostery:3.0) Gecko/125.0 Firefox/125.0",
            "sec_ch_ua": "",
            "sec_ch_ua_platform": "",
            "expected_identity": {
                "name": "Ghostery",
                "version": "3",
                "platform": "Android",
            },
        },
        {
            "actual_name": "Ghostery",
            "actual_version": "13",
            "actual_platform": "iOS",
            "user_agent": "Mozilla/5.0 (iPhone; CPU iPhone OS 17_5_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/13.0.4 Mobile/15E148 Safari/605.1.15 Ghostery Private Browser",
            "sec_ch_ua": "",
            "sec_ch_ua_platform": "",
            "expected_identity": {
                "name": "Ghostery",
                "version": "13",
                "platform": "iOS",
            },
        },
        {
            "actual_name": "Google Chrome",
            "actual_version": "126",
            "actual_platform": "Windows",
            "user_agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/126.0.0.0 Safari/537.36",
            "sec_ch_ua": '"Not/A)Brand";v="8", "Chromium";v="126", "Google Chrome";v="126"',
            "sec_ch_ua_platform": '"Windows"',
            "expected_identity": {
                "name": "Google Chrome",
                "version": "126",
                "platform": "Windows",
            },
        },
        {
            "actual_name": "Google Chrome",
            "actual_version": "126",
            "actual_platform": "macOS",
            "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/126.0.0.0 Safari/537.36",
            "sec_ch_ua": '"Not/A)Brand";v="8", "Chromium";v="126", "Google Chrome";v="126"',
            "sec_ch_ua_platform": '"macOS"',
            "expected_identity": {
                "name": "Google Chrome",
                "version": "126",
                "platform": "macOS",
            },
        },
        {
            "actual_name": "Google Chrome",
            "actual_version": "117",
            "actual_platform": "Linux",
            "user_agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/117.0.0.0 Safari/537.36",
            "sec_ch_ua": '"Google Chrome";v="117", "Not;A=Brand";v="8", "Chromium";v="117"',
            "sec_ch_ua_platform": '"Linux"',
            "expected_identity": {
                "name": "Google Chrome",
                "version": "117",
                "platform": "Linux",
            },
        },
        {
            "actual_name": "Google Chrome",
            "actual_version": "126",
            "actual_platform": "Android",
            "user_agent": "Mozilla/5.0 (Linux; Android 10; K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/126.0.0.0 Mobile Safari/537.36",
            "sec_ch_ua": '"Not/A)Brand";v="8", "Chromium";v="126", "Google Chrome";v="126"',
            "sec_ch_ua_platform": '"Android"',
            "expected_identity": {
                "name": "Google Chrome",
                "version": "126",
                "platform": "Android",
            },
        },
        {
            "actual_name": "Google Chrome",
            "actual_version": "126",
            "actual_platform": "iOS",
            "user_agent": "Mozilla/5.0 (iPhone; CPU iPhone OS 17_5 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) CriOS/126.0.6478.108 Mobile/15E148 Safari/604.1",
            "sec_ch_ua": "",
            "sec_ch_ua_platform": "",
            "expected_identity": {
                "name": "Google Chrome",
                "version": "126",
                "platform": "iOS",
            },
        },
        {
            "actual_name": "Hola",
            "actual_version": "2",
            "actual_platform": "Android",
            "user_agent": "Mozilla/5.0 (Linux; Android 14; en; SM-G991U1 Build/SP1A.210812.016) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.5060.129 HiBrowser/v2.21.5.2 UWS/ Mobile Safari/537.36",
            "sec_ch_ua": '"Android WebView";v="125", "Chromium";v="125", "Not.A/Brand";v="24"',
            "sec_ch_ua_platform": '"Android"',
            "expected_identity": {
                "name": "Hola",
                "version": "2",
                "platform": "Android",
            },
        },
        {
            "actual_name": "Mi Browser",
            "actual_version": "14",
            "actual_platform": "Android",
            "user_agent": "Mozilla/5.0 (Linux; U; Android 14; en-us; SM-G991U1 Build/UP1A.231005.007) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/112.0.5615.136 Mobile Safari/537.36 XiaoMi/MiuiBrowser/14.10.1-gn",
            "sec_ch_ua": "",
            "sec_ch_ua_platform": "",
            "expected_identity": {
                "name": "Mi Browser",
                "version": "14",
                "platform": "Android",
            },
        },
        {
            "actual_name": "Microsoft Edge",
            "actual_version": "126",
            "actual_platform": "Windows",
            "user_agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/126.0.0.0 Safari/537.36 Edg/126.0.0.0",
            "sec_ch_ua": '"Not/A)Brand";v="8", "Chromium";v="126", "Microsoft Edge";v="126"',
            "sec_ch_ua_platform": '"Windows"',
            "expected_identity": {
                "name": "Microsoft Edge",
                "version": "126",
                "platform": "Windows",
            },
        },
        {
            "actual_name": "Microsoft Edge",
            "actual_version": "126",
            "actual_platform": "macOS",
            "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/126.0.0.0 Safari/537.36 Edg/126.0.0.0",
            "sec_ch_ua": '"Not/A)Brand";v="8", "Chromium";v="126", "Microsoft Edge";v="126"',
            "sec_ch_ua_platform": '"macOS"',
            "expected_identity": {
                "name": "Microsoft Edge",
                "version": "126",
                "platform": "macOS",
            },
        },
        {
            "actual_name": "Microsoft Edge",
            "actual_version": "126",
            "actual_platform": "Linux",
            "user_agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/126.0.0.0 Safari/537.36 Edg/126.0.0.0",
            "sec_ch_ua": '"Not/A)Brand";v="8", "Chromium";v="126", "Microsoft Edge";v="126"',
            "sec_ch_ua_platform": '"Linux"',
            "expected_identity": {
                "name": "Microsoft Edge",
                "version": "126",
                "platform": "Linux",
            },
        },
        {
            "actual_name": "Microsoft Edge",
            "actual_version": "125",
            "actual_platform": "Android",
            "user_agent": "Mozilla/5.0 (Linux; Android 10; K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/125.0.0.0 Mobile Safari/537.36 EdgA/125.0.0.0",
            "sec_ch_ua": '"Microsoft Edge";v="125", "Chromium";v="125", "Not.A/Brand";v="24"',
            "sec_ch_ua_platform": '"Android"',
            "expected_identity": {
                "name": "Microsoft Edge",
                "version": "125",
                "platform": "Android",
            },
        },
        {
            "actual_name": "Microsoft Edge",
            "actual_version": "126",
            "actual_platform": "iOS",
            "user_agent": "Mozilla/5.0 (iPhone; CPU iPhone OS 17_5 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) EdgiOS/126.0.2592.67 Version/17.0 Mobile/15E148 Safari/604.1",
            "sec_ch_ua": "",
            "sec_ch_ua_platform": "",
            "expected_identity": {
                "name": "Microsoft Edge",
                "version": "126",
                "platform": "iOS",
            },
        },
        {
            "actual_name": "Opera",
            "actual_version": "111",
            "actual_platform": "Windows",
            "user_agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/125.0.0.0 Safari/537.36 OPR/111.0.0.0",
            "sec_ch_ua": '"Opera";v="111", "Chromium";v="125", "Not.A/Brand";v="24"',
            "sec_ch_ua_platform": '"Windows"',
            "expected_identity": {
                "name": "Opera",
                "version": "111",
                "platform": "Windows",
            },
        },
        {
            "actual_name": "Opera",
            "actual_version": "106",
            "actual_platform": "macOS",
            "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/120.0.0.0 Safari/537.36 OPR/106.0.0.0",
            "sec_ch_ua": '"Not_A Brand";v="8", "Chromium";v="120", "Opera";v="106"',
            "sec_ch_ua_platform": '"macOS"',
            "expected_identity": {
                "name": "Opera",
                "version": "106",
                "platform": "macOS",
            },
        },
        {
            "actual_name": "Opera",
            "actual_version": "111",
            "actual_platform": "Linux",
            "user_agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/125.0.0.0 Safari/537.36 OPR/111.0.0.0",
            "sec_ch_ua": '"Opera";v="111", "Chromium";v="125", "Not.A/Brand";v="24"',
            "sec_ch_ua_platform": '"Linux"',
            "expected_identity": {
                "name": "Opera",
                "version": "111",
                "platform": "Linux",
            },
        },
        {
            "actual_name": "Opera Mobile",
            "actual_version": "83",
            "actual_platform": "Android",
            "user_agent": "Mozilla/5.0 (Linux; Android 10; K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/126.0.0.0 Mobile Safari/537.36 OPR/83.0.0.0",
            "sec_ch_ua": '"OperaMobile";v="83", "Chromium";v="126", " Not A;Brand";v="99", "Opera";v="112"',
            "sec_ch_ua_platform": '"Android"',
            "expected_identity": {
                "name": "Opera Mobile",
                "version": "83",
                "platform": "Android",
            },
        },
        {
            "actual_name": "Opera",
            "actual_version": "4",
            "actual_platform": "iOS",
            "user_agent": "Mozilla/5.0 (iPhone; CPU iPhone OS 17_5_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/17.5 Mobile/15E148 Safari/604.1 OPT/4.7.0",
            "sec_ch_ua": "",
            "sec_ch_ua_platform": "",
            "expected_identity": {
                "name": "Opera",
                "version": "4",
                "platform": "iOS",
            },
        },
        {
            "actual_name": "Opera GX",
            "actual_version": "109",
            "actual_platform": "Windows",
            "user_agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/123.0.0.0 Safari/537.36 OPR/109.0.0.0",
            "sec_ch_ua": '"Opera GX";v="109", "Not:A-Brand";v="8", "Chromium";v="123"',
            "sec_ch_ua_platform": '"Windows"',
            "expected_identity": {
                "name": "Opera GX",
                "version": "109",
                "platform": "Windows",
            },
        },
        {
            "actual_name": "Opera GX",
            "actual_version": "109",
            "actual_platform": "macOS",
            "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/123.0.0.0 Safari/537.36 OPR/109.0.0.0",
            "sec_ch_ua": '"Opera GX";v="109", "Not:A-Brand";v="8", "Chromium";v="123"',
            "sec_ch_ua_platform": '"macOS"',
            "expected_identity": {
                "name": "Opera GX",
                "version": "109",
                "platform": "macOS",
            },
        },
        {
            "actual_name": "Opera GX",
            "actual_version": "2",
            "actual_platform": "Android",
            "user_agent": "Mozilla/5.0 (Linux; Android 14; SM-G991U1 Build/UP1A.231005.007) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/125.0.6422.165 Mobile Safari/537.36 OPX/2.3",
            "sec_ch_ua": '"Android WebView";v="125", "Chromium";v="125", "Not.A/Brand";v="24"',
            "sec_ch_ua_platform": '"Android"',
            "expected_identity": {
                "name": "Opera GX",
                "version": "2",
                "platform": "Android",
            },
        },
        {
            "actual_name": "Opera GX",
            "actual_version": "2",
            "actual_platform": "iOS",
            "user_agent": "Mozilla/5.0 (iPhone; CPU iPhone OS 17_5_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/17.5 Mobile/15E148 Safari/604.1 OPX/2.5.0",
            "sec_ch_ua": "",
            "sec_ch_ua_platform": "",
            "expected_identity": {
                "name": "Opera GX",
                "version": "2",
                "platform": "iOS",
            },
        },
        {
            "actual_name": "Pale Moon",
            "actual_version": "33",
            "actual_platform": "Windows",
            "user_agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:102.0) Gecko/20100101 Goanna/6.6 Firefox/102.0 PaleMoon/33.2.0",
            "sec_ch_ua": "",
            "sec_ch_ua_platform": "",
            "expected_identity": {
                "name": "Pale Moon",
                "version": "33",
                "platform": "Windows",
            },
        },
        {
            "actual_name": "Pale Moon",
            "actual_version": "33",
            "actual_platform": "macOS",
            "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 14.3; rv:102.0) Gecko/20100101 Goanna/6.6 Firefox/102.0 PaleMoon/33.2.0",
            "sec_ch_ua": "",
            "sec_ch_ua_platform": "",
            "expected_identity": {
                "name": "Pale Moon",
                "version": "33",
                "platform": "macOS",
            },
        },
        {
            "actual_name": "Pale Moon",
            "actual_version": "33",
            "actual_platform": "Linux",
            "user_agent": "Mozilla/5.0 (X11; Linux x86_64; rv:102.0) Gecko/20100101 Goanna/6.6 Firefox/102.0 PaleMoon/33.2.0",
            "sec_ch_ua": "",
            "sec_ch_ua_platform": "",
            "expected_identity": {
                "name": "Pale Moon",
                "version": "33",
                "platform": "Linux",
            },
        },
        {
            "actual_name": "Pluma",
            "actual_version": "125",
            "actual_platform": "Android",
            "user_agent": "Mozilla/5.0 (Linux; Android 10; K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/125.0.6422.165 Mobile Safari/537.36",
            "sec_ch_ua": '"Pluma";v="125", "Chromium";v="125", "Not.A/Brand";v="24"',
            "sec_ch_ua_platform": '"Android"',
            "expected_identity": {
                "name": "Pluma",
                "version": "125",
                "platform": "Android",
            },
        },
        {
            "actual_name": "Safari",
            "actual_version": "17",
            "actual_platform": "macOS",
            "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/17.3.1 Safari/605.1.15",
            "sec_ch_ua": "",
            "sec_ch_ua_platform": "",
            "expected_identity": {
                "name": "Safari",
                "version": "17",
                "platform": "macOS",
            },
        },
        {
            "actual_name": "Safari",
            "actual_version": "17",
            "actual_platform": "iOS",
            "user_agent": "Mozilla/5.0 (iPhone; CPU iPhone OS 17_5_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/17.5 Mobile/15E148 Safari/604.1",
            "sec_ch_ua": "",
            "sec_ch_ua_platform": "",
            "expected_identity": {
                "name": "Safari",
                "version": "17",
                "platform": "iOS",
            },
        },
        {
            "actual_name": "Samsung Internet",
            "actual_version": "25.0",
            "actual_platform": "Android",
            "user_agent": "Mozilla/5.0 (Linux; Android 10; K) AppleWebKit/537.36 (KHTML, like Gecko) SamsungBrowser/25.0 Chrome/121.0.0.0 Mobile Safari/537.36",
            "sec_ch_ua": '"Not A(Brand";v="99", "Samsung Internet";v="25.0", "Chromium";v="121"',
            "sec_ch_ua_platform": '"Android"',
            "expected_identity": {
                "name": "Samsung Internet",
                "version": "25.0",
                "platform": "Android",
            },
        },
        {
            "actual_name": "SeaMonkey",
            "actual_version": "2",
            "actual_platform": "Windows",
            "user_agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:91.0) Gecko/20100101 Firefox/91.0 SeaMonkey/2.53.18.2",
            "sec_ch_ua": "",
            "sec_ch_ua_platform": "",
            "expected_identity": {
                "name": "SeaMonkey",
                "version": "2",
                "platform": "Windows",
            },
        },
        {
            "actual_name": "SeaMonkey",
            "actual_version": "2",
            "actual_platform": "macOS",
            "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 14.3; rv:91.0) Gecko/20100101 Firefox/91.0 SeaMonkey/2.53.18.2",
            "sec_ch_ua": "",
            "sec_ch_ua_platform": "",
            "expected_identity": {
                "name": "SeaMonkey",
                "version": "2",
                "platform": "macOS",
            },
        },
        {
            "actual_name": "SeaMonkey",
            "actual_version": "2",
            "actual_platform": "Linux",
            "user_agent": "Mozilla/5.0 (X11; Linux x86_64; rv:91.0) Gecko/20100101 Firefox/91.0 SeaMonkey/2.53.18.2",
            "sec_ch_ua": "",
            "sec_ch_ua_platform": "",
            "expected_identity": {
                "name": "SeaMonkey",
                "version": "2",
                "platform": "Linux",
            },
        },
        {
            "actual_name": "Yandex Browser",
            "actual_version": "24",
            "actual_platform": "iOS",
            "user_agent": "Mozilla/5.0 (iPhone; CPU iPhone OS 17_5 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/17.0 YaBrowser/24.6.3.341.10 SA/3 Mobile/15E148 Safari/604.1",
            "sec_ch_ua": "",
            "sec_ch_ua_platform": "",
            "expected_identity": {
                "name": "Yandex Browser",
                "version": "24",
                "platform": "iOS",
            },
        },
        # Cannot be correctly identified
        {
            "actual_name": "Arc",
            "actual_version": "1.48.2",
            "actual_platform": "macOS",
            "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/126.0.0.0 Safari/537.36",
            "sec_ch_ua": '"Not/A)Brand";v="8", "Chromium";v="126"',
            "sec_ch_ua_platform": '"macOS"',
            "expected_identity": {
                "name": "Chromium-based browser",
                "version": "126",
                "platform": "macOS",
            },
        },
        {
            "actual_name": "Brave",
            "actual_version": "1.66.2",
            "actual_platform": "iOS",
            "user_agent": "Mozilla/5.0 (iPhone; CPU iPhone OS 17_5 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/17.5 Mobile/15E148 Safari/604.1",
            "sec_ch_ua": "",
            "sec_ch_ua_platform": "",
            "expected_identity": {
                "name": "Safari",
                "version": "17",
                "platform": "iOS",
            },
        },
        {
            "actual_name": "Dolphin",
            "actual_version": "12.4.1",
            "actual_platform": "Android",
            "user_agent": "Mozilla/5.0 (Linux; Android 14; SM-G991U1 Build/UP1A.231005.007; wv) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/125.0.6422.165 Mobile Safari/537.36",
            "sec_ch_ua": '"Android WebView";v="125", "Chromium";v="125", "Not.A/Brand";v="24"',
            "sec_ch_ua_platform": '"Android"',
            "expected_identity": {
                "name": "Chromium-based browser",
                "version": "125",
                "platform": "Android",
            },
        },
        {
            "actual_name": "Ecosia",
            "actual_version": "123.0.6312.21",
            "actual_platform": "Windows",
            "user_agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/123.0.0.0 Safari/537.36",
            "sec_ch_ua": '"Chromium";v="123", "Not:A-Brand";v="8"',
            "sec_ch_ua_platform": '"Windows"',
            "expected_identity": {
                "name": "Chromium-based browser",
                "version": "123",
                "platform": "Windows",
            },
        },
        {
            "actual_name": "Ecosia",
            "actual_version": "123.0.6312.22",
            "actual_platform": "macOS",
            "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/123.0.0.0 Safari/537.36",
            "sec_ch_ua": '"Chromium";v="123", "Not:A-Brand";v="8"',
            "sec_ch_ua_platform": '"macOS"',
            "expected_identity": {
                "name": "Chromium-based browser",
                "version": "123",
                "platform": "macOS",
            },
        },
        {
            "actual_name": "Firefox Focus",
            "actual_version": "127",
            "actual_platform": "Android",
            "user_agent": "Mozilla/5.0 (Android 14; Mobile; rv:127.0) Gecko/127.0 Firefox/127.0",
            "sec_ch_ua": "",
            "sec_ch_ua_platform": "",
            "expected_identity": {
                "name": "Firefox",
                "version": "127",
                "platform": "Android",
            },
        },
        {
            "actual_name": "Floorp",
            "actual_version": "11.14.1",
            "actual_platform": "Windows",
            "user_agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:109.0) Gecko/20100101 Firefox/115.0",
            "sec_ch_ua": "",
            "sec_ch_ua_platform": "",
            "expected_identity": {
                "name": "Firefox",
                "version": "115",
                "platform": "Windows",
            },
        },
        {
            "actual_name": "Floorp",
            "actual_version": "11",
            "actual_platform": "macOS",
            "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.15; rv:109.0) Gecko/20100101 Firefox/115.0",
            "sec_ch_ua": "",
            "sec_ch_ua_platform": "",
            "expected_identity": {
                "name": "Firefox",
                "version": "115",
                "platform": "macOS",
            },
        },
        {
            "actual_name": "Floorp",
            "actual_version": "11.14.1",
            "actual_platform": "Linux",
            "user_agent": "Mozilla/5.0 (X11; Linux x86_64; rv:109.0) Gecko/20100101 Firefox/115.0",
            "sec_ch_ua": "",
            "sec_ch_ua_platform": "",
            "expected_identity": {
                "name": "Firefox",
                "version": "115",
                "platform": "Linux",
            },
        },
        {
            "actual_name": "Ghostery",
            "actual_version": "2024.05.1",
            "actual_platform": "Windows",
            "user_agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:126.0) Gecko/20100101 Firefox/126.0",
            "sec_ch_ua": "",
            "sec_ch_ua_platform": "",
            "expected_identity": {
                "name": "Firefox",
                "version": "126",
                "platform": "Windows",
            },
        },
        {
            "actual_name": "Ghostery",
            "actual_version": "2024.05.1",
            "actual_platform": "macOS",
            "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.15; rv:126.0) Gecko/20100101 Firefox/126.0",
            "sec_ch_ua": "",
            "sec_ch_ua_platform": "",
            "expected_identity": {
                "name": "Firefox",
                "version": "126",
                "platform": "macOS",
            },
        },
        {
            "actual_name": "Ghostery",
            "actual_version": "2024.05.1",
            "actual_platform": "Linux",
            "user_agent": "Mozilla/5.0 (X11; Linux x86_64; rv:126.0) Gecko/20100101 Firefox/126.0",
            "sec_ch_ua": "",
            "sec_ch_ua_platform": "",
            "expected_identity": {
                "name": "Firefox",
                "version": "126",
                "platform": "Linux",
            },
        },
        {
            "actual_name": "Iridium",
            "actual_version": "116.0",
            "actual_platform": "Windows",
            "user_agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/116.0.5845.140 Safari/537.36",
            "sec_ch_ua": '"Not)A;Brand";v="24", "Chromium";v="116"',
            "sec_ch_ua_platform": '"Windows"',
            "expected_identity": {
                "name": "Chromium-based browser",
                "version": "116",
                "platform": "Windows",
            },
        },
        {
            "actual_name": "Iridium",
            "actual_version": "116",
            "actual_platform": "macOS",
            "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/116.0.5845.140 Safari/537.36",
            "sec_ch_ua": '"Not)A;Brand";v="24", "Chromium";v="116"',
            "sec_ch_ua_platform": '"macOS"',
            "expected_identity": {
                "name": "Chromium-based browser",
                "version": "116",
                "platform": "macOS",
            },
        },
        {
            "actual_name": "Kiwi Browser",
            "actual_version": "124.0.6327.4",
            "actual_platform": "Android",
            "user_agent": "Mozilla/5.0 (Linux; Android 10; K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/124.0.0.0 Mobile Safari/537.36",
            "sec_ch_ua": '"Not-A.Brand";v="99", "Chromium";v="124"',
            "sec_ch_ua_platform": '"Android"',
            "expected_identity": {
                "name": "Chromium-based browser",
                "version": "124",
                "platform": "Android",
            },
        },
        {
            "actual_name": "LibreWolf",
            "actual_version": "127",
            "actual_platform": "Windows",
            "user_agent": "Mozilla/5.0 (Windows NT 10.0; rv:127.0) Gecko/20100101 Firefox/127.0",
            "sec_ch_ua": "",
            "sec_ch_ua_platform": "",
            "expected_identity": {
                "name": "Firefox",
                "version": "127",
                "platform": "Windows",
            },
        },
        {
            "actual_name": "LibreWolf",
            "actual_version": "127",
            "actual_platform": "macOS",
            "user_agent": "Mozilla/5.0 (Windows NT 10.0; rv:127.0) Gecko/20100101 Firefox/127.0",
            "sec_ch_ua": "",
            "sec_ch_ua_platform": "",
            "expected_identity": {
                "name": "Firefox",
                "version": "127",
                "platform": "Windows",
            },
        },
        {
            "actual_name": "LibreWolf",
            "actual_version": "127",
            "actual_platform": "Linux",
            "user_agent": "Mozilla/5.0 (Windows NT 10.0; rv:127.0) Gecko/20100101 Firefox/127.0",
            "sec_ch_ua": "",
            "sec_ch_ua_platform": "",
            "expected_identity": {
                "name": "Firefox",
                "version": "127",
                "platform": "Windows",
            },
        },
        {
            "actual_name": "Maxthon",
            "actual_version": "7.1.8.9001",
            "actual_platform": "Windows",
            "user_agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/109.0.0.0 Safari/537.36",
            "sec_ch_ua": "",
            "sec_ch_ua_platform": "",
            "expected_identity": {
                "name": "Google Chrome",
                "version": "109",
                "platform": "Windows",
            },
        },
        {
            "actual_name": "Maxthon",
            "actual_version": "7",
            "actual_platform": "macOS",
            "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/109.0.0.0 Safari/537.36",
            "sec_ch_ua": "",
            "sec_ch_ua_platform": "",
            "expected_identity": {
                "name": "Google Chrome",
                "version": "109",
                "platform": "macOS",
            },
        },
        {
            "actual_name": "Maxthon",
            "actual_version": "7.4.3.800",
            "actual_platform": "Android",
            "user_agent": "Mozilla/5.0 (Linux; Android 14; SM-G991U1 Build/UP1A.231005.007) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/125.0.6422.165 Mobile Safari/537.36",
            "sec_ch_ua": '"Android WebView";v="125", "Chromium";v="125", "Not.A/Brand";v="24"',
            "sec_ch_ua_platform": '"Android"',
            "expected_identity": {
                "name": "Chromium-based browser",
                "version": "125",
                "platform": "Android",
            },
        },
        {
            "actual_name": "Maxthon",
            "actual_version": "7.3.5",
            "actual_platform": "iOS",
            "user_agent": "Mozilla/5.0 (iPhone; CPU iPhone OS 17_5_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/17.4 Mobile/15E148 Safari/605.1.15",
            "sec_ch_ua": "",
            "sec_ch_ua_platform": "",
            "expected_identity": {
                "name": "Safari",
                "version": "17",
                "platform": "iOS",
            },
        },
        {
            "actual_name": "Phoenix",
            "actual_version": "21",
            "actual_platform": "iOS",
            "user_agent": "Mozilla/5.0 (iPhone; CPU iPhone OS 17_5_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/17.0 Mobile/15E148 Safari/604.1",
            "sec_ch_ua": "",
            "sec_ch_ua_platform": "",
            "expected_identity": {
                "name": "Safari",
                "version": "17",
                "platform": "iOS",
            },
        },
        {
            "actual_name": "Puma",
            "actual_version": "3.7.16",
            "actual_platform": "Android",
            "user_agent": "Mozilla/5.0 (Android 14; Mobile; rv:124.0) Gecko/124.0 Firefox/124.0",
            "sec_ch_ua": "",
            "sec_ch_ua_platform": "",
            "expected_identity": {
                "name": "Firefox",
                "version": "124",
                "platform": "Android",
            },
        },
        {
            "actual_name": "Puma",
            "actual_version": "3.8.18",
            "actual_platform": "iOS",
            "user_agent": "Mozilla/5.0 (iPhone; CPU iPhone OS 17_5_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) FxiOS/36.0  Mobile/15E148 Safari/605.1.15",
            "sec_ch_ua": "",
            "sec_ch_ua_platform": "",
            "expected_identity": {
                "name": "Firefox",
                "version": "36",
                "platform": "iOS",
            },
        },
        {
            "actual_name": "RedApp",
            "actual_version": "3.4.4",
            "actual_platform": "iOS",
            "user_agent": "Mozilla/5.0 (iPhone; CPU iPhone OS 17_0_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/17.0 Mobile/15E148 Safari/604.1",
            "sec_ch_ua": "",
            "sec_ch_ua_platform": "",
            "expected_identity": {
                "name": "Safari",
                "version": "17",
                "platform": "iOS",
            },
        },
        {
            "actual_name": "Sidekick",
            "actual_version": "120.58.1.38488",
            "actual_platform": "Windows",
            "user_agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/122.0.0.0 Safari/537.36",
            "sec_ch_ua": '"Not_A Brand";v="8", "Chromium";v="120", "Google Chrome";v="120"',
            "sec_ch_ua_platform": '"Windows"',
            "expected_identity": {
                "name": "Google Chrome",
                "version": "120",
                "platform": "Windows",
            },
        },
        {
            "actual_name": "Sidekick",
            "actual_version": "122.60.1.40404",
            "actual_platform": "macOS",
            "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/125.0.0.0 Safari/537.36",
            "sec_ch_ua": '"Chromium";v="122", "Not(A:Brand";v="24", "Google Chrome";v="122"',
            "sec_ch_ua_platform": '"macOS"',
            "expected_identity": {
                "name": "Google Chrome",
                "version": "122",
                "platform": "macOS",
            },
        },
        {
            "actual_name": "Sidekick",
            "actual_version": "122.60.1.39613",
            "actual_platform": "Linux",
            "user_agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/125.0.0.0 Safari/537.36",
            "sec_ch_ua": '"Chromium";v="122", "Not(A:Brand";v="24", "Google Chrome";v="122"',
            "sec_ch_ua_platform": '"Linux"',
            "expected_identity": {
                "name": "Google Chrome",
                "version": "122",
                "platform": "Linux",
            },
        },
        {
            "actual_name": "Thorium",
            "actual_version": "124.0.6367.218",
            "actual_platform": "Windows",
            "user_agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/124.0.0.0 Safari/537.36",
            "sec_ch_ua": '"Not-A.Brand";v="99", "Chromium";v="124"',
            "sec_ch_ua_platform": '"Windows"',
            "expected_identity": {
                "name": "Chromium-based browser",
                "version": "124",
                "platform": "Windows",
            },
        },
        {
            "actual_name": "Thorium",
            "actual_version": "124.0.6367.218",
            "actual_platform": "macOS",
            "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/124.0.0.0 Safari/537.36",
            "sec_ch_ua": '"Not-A.Brand";v="99", "Chromium";v="124"',
            "sec_ch_ua_platform": '"macOS"',
            "expected_identity": {
                "name": "Chromium-based browser",
                "version": "124",
                "platform": "macOS",
            },
        },
        {
            "actual_name": "Thorium",
            "actual_version": "124.0.6367.218",
            "actual_platform": "Linux",
            "user_agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/124.0.0.0 Safari/537.36",
            "sec_ch_ua": '"Not-A.Brand";v="99", "Chromium";v="124"',
            "sec_ch_ua_platform": '"Linux"',
            "expected_identity": {
                "name": "Chromium-based browser",
                "version": "124",
                "platform": "Linux",
            },
        },
        {
            "actual_name": "Vivaldi",
            "actual_version": "6.8.3381.44",
            "actual_platform": "Windows",
            "user_agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/126.0.0.0 Safari/537.36",
            "sec_ch_ua": '"Not/A)Brand";v="8", "Chromium";v="126", "Google Chrome";v="126"',
            "sec_ch_ua_platform": '"Windows"',
            "expected_identity": {
                "name": "Google Chrome",
                "version": "126",
                "platform": "Windows",
            },
        },
        {
            "actual_name": "Vivaldi",
            "actual_version": "6.5.3206.55",
            "actual_platform": "macOS",
            "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/120.0.0.0 Safari/537.36",
            "sec_ch_ua": '"Not_A Brand";v="8", "Chromium";v="120", "Google Chrome";v="120"',
            "sec_ch_ua_platform": '"macOS"',
            "expected_identity": {
                "name": "Google Chrome",
                "version": "120",
                "platform": "macOS",
            },
        },
        {
            "actual_name": "Vivaldi",
            "actual_version": "6.8.3381.44",
            "actual_platform": "Linux",
            "user_agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/126.0.0.0 Safari/537.36",
            "sec_ch_ua": '"Not/A)Brand";v="8", "Chromium";v="126", "Google Chrome";v="126"',
            "sec_ch_ua_platform": '"Linux"',
            "expected_identity": {
                "name": "Google Chrome",
                "version": "126",
                "platform": "Linux",
            },
        },
        {
            "actual_name": "Vivaldi",
            "actual_version": "6.5.3217.110",
            "actual_platform": "Android",
            "user_agent": "Mozilla/5.0 (Linux; Android 10; K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/120.0.0.0 Mobile Safari/537.36",
            "sec_ch_ua": '"Not_A Brand";v="8", "Chromium";v="120", "Google Chrome";v="120"',
            "sec_ch_ua_platform": '"Android"',
            "expected_identity": {
                "name": "Google Chrome",
                "version": "120",
                "platform": "Android",
            },
        },
        {
            "actual_name": "Vivaldi",
            "actual_version": "6.7.3335.57",
            "actual_platform": "iOS",
            "user_agent": "Mozilla/5.0 (iPhone; CPU iPhone OS 17_5 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) CriOS/124.0.6367.166 Mobile/15E148 Safari/604.1",
            "sec_ch_ua": "",
            "sec_ch_ua_platform": "",
            "expected_identity": {
                "name": "Google Chrome",
                "version": "124",
                "platform": "iOS",
            },
        },
        {
            "actual_name": "Waterfox",
            "actual_version": "G6.0.16",
            "actual_platform": "Windows",
            "user_agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:109.0) Gecko/20100101 Firefox/115.0",
            "sec_ch_ua": "",
            "sec_ch_ua_platform": "",
            "expected_identity": {
                "name": "Firefox",
                "version": "115",
                "platform": "Windows",
            },
        },
        {
            "actual_name": "Waterfox",
            "actual_version": "G6.0.16",
            "actual_platform": "macOS",
            "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.15; rv:109.0) Gecko/20100101 Firefox/115.0",
            "sec_ch_ua": "",
            "sec_ch_ua_platform": "",
            "expected_identity": {
                "name": "Firefox",
                "version": "115",
                "platform": "macOS",
            },
        },
        {
            "actual_name": "Waterfox",
            "actual_version": "G6.0.16",
            "actual_platform": "Linux",
            "user_agent": "Mozilla/5.0 (X11; Linux x86_64; rv:109.0) Gecko/20100101 Firefox/115.0",
            "sec_ch_ua": "",
            "sec_ch_ua_platform": "",
            "expected_identity": {
                "name": "Firefox",
                "version": "115",
                "platform": "Linux",
            },
        },
        {
            "actual_name": "Waterfox",
            "actual_version": "1.0.8.2",
            "actual_platform": "Android",
            "user_agent": "Mozilla/5.0 (Android 14; Mobile; rv:127.0) Gecko/127.0 Firefox/127.0",
            "sec_ch_ua": "",
            "sec_ch_ua_platform": "",
            "expected_identity": {
                "name": "Firefox",
                "version": "127",
                "platform": "Android",
            },
        },
    ]

    for browser in browsers:
        browser_identity = identify_browser(
            browser["user_agent"],
            browser["sec_ch_ua"],
            browser["sec_ch_ua_platform"],
        )

        assert browser_identity == browser["expected_identity"]
